Bases = {}

local selectedBase = 0
local timer = 1
local grid_size = 128
local grids_X = window_width / grid_size
local grids_Y = window_height / grid_size
local pulse = -.1
local bases = {}
local lines = {}

function Bases:distance ( x1, y1, x2, y2 )
  local dx = x1 - x2
  local dy = y1 - y2
  return math.sqrt ( dx * dx + dy * dy )
end

function Bases:load()
  for y=2, grids_Y do
    for x=1, grids_X do
      randomX = love.math.random(24, grid_size - 24)
      randomY = love.math.random(24, grid_size - 24)
      n = #bases + 1
      bases[n] = {}
      bases[n].id = n
      bases[n].hovering = false
      bases[n].selected = false
      bases[n].specialty = ""
      bases[n].units = 100
      bases[n].lineWidth =2
      bases[n].owner = 0
      bases[n].neighbors = {}
      bases[n].lines = {}
      bases[n].color = {0,0,255,255}
      bases[n].body =  love.physics.newBody(world, (x * grid_size) - grid_size + randomX, (y * grid_size) - grid_size + randomY, "dynamic")
      bases[n].shape = love.physics.newCircleShape(6)
      bases[n].fixture = love.physics.newFixture(bases[n].body, bases[n].shape)
      if n > 1 then
        for i = n - 1, 1, -1 do
          distance = Bases:distance(bases[n].body:getX(), bases[n].body:getY(), bases[i].body:getX(), bases[i].body:getY())
          --print(distance)
          if distance < 180 then -- how close do they need to be to be considered a "neighbor" and attackable?
            x = #lines + 1
            lines[x] = {}
            lines[x].body = love.physics.newBody(world, bases[n].body:getX(), bases[n].body:getY(), "dynamic")
            lines[x].shape = love.physics.newChainShape(false, bases[n].body:getX(), bases[n].body:getY(), bases[i].body:getX(), bases[i].body:getY())
            lines[x].fixture = love.physics.newFixture(lines[x].body, lines[x].shape, 1)
            table.insert(bases[i].neighbors,1,n)
            table.insert(bases[i].lines,1,x)
          end
        end
      end
    end
  end
  local randomBase = love.math.random(1, #bases)
  print(randomBase)
  bases[randomBase].owner = 1
  bases[randomBase].selected = true
  bases[randomBase].specialty = "capitol"
  selectedBase = randomBase
end

function Bases:update(dt)
  for k,base in pairs(bases) do
    if base.owner == 1 then
      if timer <= 0 then
        base.units = base.units + 1
        timer = 1
      else
        timer = timer - dt
      end
      -- How should I handle the unit increases for the NPC dummy bases?
    --[[elseif base.owner == 0  
      if timer <= 0 then
        base.units = base.units + 1
        timer = 1
      else
        timer = timer - dt /3
      end]]--
    end
  end
end

function Bases:draw()
  love.graphics.push()
  local tx = math.floor(-bases[selectedBase].body:getX() + window_width / 2)
  local ty = math.floor(-bases[selectedBase].body:getY() + window_height / 2)
  love.graphics.translate(tx, ty)
  love.graphics.setColor(255,255,255,128)
  love.graphics.setLineWidth(2)
  for k,base in pairs(bases) do    
    local mouseOver = base.shape:testPoint( base.body:getX(), base.body:getY(), 0, love.mouse.getX() - tx, love.mouse.getY() - ty )
    if mouseOver then
      base.hovering = true
      Bases:pulsate(base.id)
      if love.mouse.isDown(1) then
        base.owner = 1
        selectedBase = base.id
      end      
    else
      base.hovering = false
    end
    if base.owner == 1 then
      love.graphics.setColor(128,128,128,255)
      for k,line in pairs(base.lines) do
        love.graphics.line(lines[line].shape:getPoints())
      end
      love.graphics.setColor(0,0,255,192)
      love.graphics.setLineWidth(base.lineWidth)
      Bases:drawBase(base.id)
      for i,j in pairs(base.neighbors) do -- find and draw the neighboring bases
        -- print("Key (value): " .. i .. "(" ..j .. ")")
        love.graphics.setColor(255,255,255,255)
        love.graphics.setLineWidth(bases[j].lineWidth)
        Bases:drawBase(bases[j].id)
      end
    else love.graphics.setColor(255,255,255,255) end
  end 
  love.graphics.pop()
end

function Bases:drawBase(id)
  love.graphics.circle("line", bases[id].body:getX(), bases[id].body:getY(), 12)
  love.graphics.circle("fill", bases[id].body:getX(), bases[id].body:getY(), 6)
  love.graphics.setColor(255,255,255,255)
  love.graphics.print(bases[id].units, bases[id].body:getX() + 14, bases[id].body:getY() - 14)
end

function Bases:pulsate(id)
  if bases[id].lineWidth < 1 or bases[id].lineWidth > 3 then
    pulse = pulse * -1
  end
  bases[id].lineWidth = bases[id].lineWidth + pulse
end

return Bases